Package: tabstats
Type: Package
Title: A Lightweight Toolkit for Displaying Customizable Tables
Version: 0.2.1
Authors@R: c(
    person(
        "Joshua", "Marie",
        email = "joshua.marie.k@gmail.com",
        role = c("aut", "cre")
    )
    )
Maintainer: Joshua Marie <joshua.marie.k@gmail.com>
Description: A lightweight toolkit that provides functions for printing tables from
    input data in the R console or terminal with customizable formatting. Supported
    outputs include American Psychological Association (APA)-style tables
    (American Psychological Association, 2020, ISBN:9781433832178),
    correlation matrices, contingency tables, and two-column summary tables.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.1.0)
URL: https://github.com/s7-stats/tabstats
BugReports: https://github.com/s7-stats/tabstats/issues
Imports: cli, tibble, vctrs, utils, stats
Suggests: broom, rstatix, glue, tidyr, testthat (>= 3.0.0), knitr,
        rmarkdown, fansi, crayon, dplyr
Config/testthat/edition: 3
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-07-08 10:21:52 UTC; DESKTOP
Author: Joshua Marie [aut, cre]
Repository: CRAN
Date/Publication: 2026-07-08 10:50:02 UTC
Built: R 4.4.3; ; 2026-07-08 15:13:41 UTC; unix
