# Generated by roxygen2: do not edit by hand

S3method(format,tabstats_style)
S3method(print,tabstats_style)
export(cm_style)
export(corr_matrix)
export(cross_table)
export(ct_style)
export(new_corr_data)
export(new_pairwise_data)
export(pairwise_matrix)
export(sm_style)
export(table_default)
export(table_summary)
export(tabstats_options)
export(td_style)
