% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/style-helpers.R
\name{sm_style}
\alias{sm_style}
\title{Style specification for table_summary()}
\usage{
sm_style(
  left_col = NULL,
  right_col = NULL,
  border_text = NULL,
  title = NULL,
  sep = NULL
)
}
\arguments{
\item{left_col}{Style for the left column. A string (e.g. \code{"blue_bold"})
or a function \verb{\\(x) ...}.}

\item{right_col}{Style for the right column.}

\item{border_text}{Style for the horizontal border lines.}

\item{title}{Style for the title text.}

\item{sep}{A single character used as the column separator (e.g. \code{"|"}).}
}
\value{
An object of class \code{c("sm_style", "tabstats_style")}.
}
\description{
Constructs a validated style object for use with \code{\link[=table_summary]{table_summary()}}.
}
\examples{
sm_style(left_col = "blue_bold", right_col = "green", title = "bold")
sm_style(left_col = \(x) cli::col_red(x), sep = "|")

}
