% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/style-helpers.R
\name{ct_style}
\alias{ct_style}
\title{Style specification for cross_table()}
\usage{
ct_style(
  observed = NULL,
  expected = NULL,
  row_percentage = NULL,
  col_percentage = NULL,
  total_percentage = NULL,
  total = NULL,
  title = NULL,
  border_text = NULL
)
}
\arguments{
\item{observed}{Style for observed frequency cells.}

\item{expected}{Style for expected frequency values.}

\item{row_percentage}{Style for row percentage values.}

\item{col_percentage}{Style for column percentage values.}

\item{total_percentage}{Style for grand total percentage values.}

\item{total}{Style for total row/column cells.}

\item{title}{Style for the table title.}

\item{border_text}{Style for the horizontal border lines.}
}
\value{
An object of class \code{c("ct_style", "tabstats_style")}.
}
\description{
Constructs a validated style object for use with \code{\link[=cross_table]{cross_table()}}.
}
\examples{
ct_style(observed = "blue", expected = "yellow", title = "bold")
ct_style(observed = \(ctx) cli::col_green(ctx$formatted_text))

}
