% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/style-helpers.R
\name{td_style}
\alias{td_style}
\title{Style specification for table_default()}
\usage{
td_style(...)
}
\arguments{
\item{...}{Named style entries. Each name must be a column name, a column
index as a string (e.g. \code{"1"}), or \code{"title"}. Each value is a string or
a function \verb{\\(ctx) ...} where \code{ctx} is a context list.}
}
\value{
An object of class \code{c("td_style", "tabstats_style")}.
}
\description{
Constructs a validated style object for use with \code{\link[=table_default]{table_default()}}.
}
\examples{
td_style(mpg = "cyan", cyl = "magenta")
td_style(mpg = \(ctx) cli::col_red(ctx$value), title = "bold")

}
